﻿using ClothShopContracts.BindingModels;
using ClothShopContracts.SearchModels;
using ClothShopContracts.StoragesContracts;
using ClothShopContracts.ViewModels;
using ClothShopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ClothShopDatabaseImplement.Implements
{
    public class TextileStorage : ITextileStorage
    {
        public List<TextileViewModel> GetFullList()
        {
            using var context = new ClothShopDatabase();
            return context.Textiles
             .Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<TextileViewModel> GetFilteredList(TextileSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TextileName))
            {
                return new();
            }
            using var context = new ClothShopDatabase();
            return context.Textiles
             .Include(x => x.Components)
            .ThenInclude(x => x.Component)
            .Where(x => x.TextileName.Contains(model.TextileName))
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public TextileViewModel? GetElement(TextileSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TextileName) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new ClothShopDatabase();
            return context.Textiles
             .Include(x => x.Components)
             .ThenInclude(x => x.Component)
             .FirstOrDefault(x => (!string.IsNullOrEmpty(model.TextileName) &&
            x.TextileName == model.TextileName) ||
            (model.Id.HasValue && x.Id ==
            model.Id))
            ?.GetViewModel;
        }
        public TextileViewModel? Insert(TextileBindingModel model)
        {
            using var context = new ClothShopDatabase();
            var newTextile = Textile.Create(context, model);
            if (newTextile == null)
            {
                return null;
            }
            context.Textiles.Add(newTextile);
            context.SaveChanges();
            return newTextile.GetViewModel;
        }
        public TextileViewModel? Update(TextileBindingModel model)
        {
            using var context = new ClothShopDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var Textile = context.Textiles.FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (Textile == null)
                {
                    return null;
                }
                Textile.Update(model);
                context.SaveChanges();
                Textile.UpdateComponents(context, model);
                transaction.Commit();
                return Textile.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public TextileViewModel? Delete(TextileBindingModel model)
        {
            using var context = new ClothShopDatabase();
            var element = context.Textiles
            .Include(x => x.Components)
            .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Textiles.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

    }
}
