﻿using ClothShopContracts.BindingModels;
using ClothShopContracts.ViewModels;
using ClothShopDataModels.Enums;

using System.ComponentModel.DataAnnotations;

namespace ClothShopDatabaseImplement.Models
{
    public class Order
    {
        public int Id { get; private set; }
        [Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; }
        [Required]
        public DateTime DateCreate { get; private set; }
        public DateTime? DateImplement { get; private set; }
        [Required]
        public int TextileId { get; private set; }
        public virtual Textile? Textiles { get; private set; }

        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                TextileId = model.TextileId,
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            TextileId = TextileId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            Id = Id,
            TextileName = Textiles?.TextileName ?? string.Empty
        };
    }
}
