﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ClothShopContracts.BindingModels;
using ClothShopContracts.BusinessLogicContracts;
using ClothShopContracts.ViewModels;
using ClothShopContracts.SearchModels;
using Microsoft.Extensions.Logging;

namespace ClothShopView
{
    public partial class FormCreateOrder : Form
    {
        private readonly ILogger _logger;
        private readonly ITextileLogic _logicP;
        private readonly IOrderLogic _logicO;
        private List<TextileViewModel>? _list;
        public FormCreateOrder()
        {
            InitializeComponent();
        }
        public FormCreateOrder(ILogger<FormCreateOrder> logger, ITextileLogic logicP, IOrderLogic logicO)
        {
            InitializeComponent();
            _logger = logger;
            _logicP = logicP;
            _logicO = logicO;
        }
        private void FormCreateOrder_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка изделий для заказа");
            _list = _logicP.ReadList(null);
            if (_list != null)
            {
                comboBoxTextile.DisplayMember = "TextileName";
                comboBoxTextile.ValueMember = "Id";
                comboBoxTextile.DataSource = _list;
                comboBoxTextile.SelectedItem = null;
                _logger.LogInformation("Загрузка суши для заказа");
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxTextile.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание заказа");
            try
            {
                var operationResult = _logicO.CreateOrder(new OrderBindingModel
                {
                    TextileId = Convert.ToInt32(comboBoxTextile.SelectedValue),
                    Count = Convert.ToInt32(textBoxCount.Text),
                    Sum = Convert.ToDouble(textBoxSum.Text)
                });
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании заказа. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение",
               MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
               MessageBoxIcon.Error);
            }
        }
        private void CalcSum()
        {
            if (comboBoxTextile.SelectedValue != null &&
           !string.IsNullOrEmpty(textBoxCount.Text))
            {
                try
                {
                    int id = Convert.ToInt32(comboBoxTextile.SelectedValue);
                    var textile = _logicP.ReadElement(new TextileSearchModel
                    {
                        Id = id
                    });
                    int count = Convert.ToInt32(textBoxCount.Text);
                    textBoxSum.Text = Math.Round(count * (textile?.Price ?? 0),
                    2).ToString();
                    _logger.LogInformation("Расчет суммы заказа");
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка расчета суммы заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }

        private void comboBoxTextile_SelectedIndexChanged(object sender, EventArgs e)
        {
            CalcSum();
        }

        private void textBoxPrice_TextChanged(object sender, EventArgs e)
        {
            CalcSum();
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
