using ClothShopContracts.BusinessLogicContracts;
using ClothShopContracts.StoragesContracts;
using ClothShopBusinessLogic.BusinessLogics;
using ClothShopDatabaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;

using System;
using System.Drawing;
using ClothShopBusinessLogic.OfficePackage;
using AbstractShopBusinessLogic.OfficePackage;
using ClothShopBusinessLogic.OfficePackage.Implements;
using AbstractShopBusinessLogic.OfficePackage.Implements;

namespace ClothShopView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<ITextileStorage, TextileStorage>();
            services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<ITextileLogic, TextileLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<FormMain>();
            services.AddTransient<FormComponent>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormTextile>();
            services.AddTransient<FormTextileComponent>();
            services.AddTransient<FormTextiles>();
            services.AddTransient<FormReportTextileComponents>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
        }
    }
}